import unittest
from src.String_Calculator import StringCalculator

class TestStringCalculator(unittest.TestCase):
    def setUp(self):
        self.c = StringCalculator()

    def test_empty(self):
        res = self.c.add("")
        self.assertEqual(res, 0)

    def test_oneNumber(self):
        res = self.c.add("1")
        self.assertEqual(res, 1)

    def test_addingTwoNumbers(self):
        res = self.c.add("1,2")
        self.assertEqual(res, 3)

    def test_addingTwoNumbersWithZero(self):
        res = self.c.add("0,5")
        self.assertEqual(res, 5)

    def test_handleFloat(self):
        res = self.c.add("3.5")
        self.assertEqual(res, "only integers allowed")

    def test_handleLetter(self):
        res = self.c.add("1, z")
        self.assertEqual(res, "only integers allowed")

    def test_addWithBackslashN(self):
        res = self.c.add("1\n2,3")
        self.assertEqual(res, 6)

    def test_negativeValues(self):
        res = self.c.add("-3")
        self.assertEqual(res, "negatives not allowed")

    def test_delimiter(self):
        res = self.c.add("//;\n1;2")
        self.assertEqual(res, 3)

    def test_thousandone(self):
        res = self.c.add("2, 1001")
        self.assertEqual(res, 2)

    def test_multidelimiter(self):
        res = self.c.add("//[***]\n1***2***3")
        self.assertEqual(res, 6)

    def test_multi_negative(self):
        res = self.c.add("-3, -4")
        self.assertEqual(res, "negatives not allowed " + str([-3, -4]))

    def test_space_between_numbers(self):
        res = self.c.add(" 4  , 5")
        self.assertEqual(res, 9)

    def test_multiple_num_with_thousandone(self):
        res = self.c.add(" 2, 1001, 5")
        self.assertEqual(res, 7)

    def test_empty_text(self):
        res = self.c.add("//;\n")
        self.assertEqual(res, 0)

    def test_one_number_with_empty_string(self):
        res = self.c.add("1,")
        self.assertEqual(res, 1)

    def test_negative_with_positive(self):
        res = self.c.add("-2, 5")
        self.assertEqual(res, "negatives not allowed")

    def test_mixture(self):
        res = self.c.add("//;\n-1;2;1001;-3")
        self.assertEqual(res, "negatives not allowed " + str([-1, -3]))


if __name__ == "__main__":
    unittest.main()