import re
from src.interfaces import ICalculator


class Calculator(ICalculator):
    """Ein Taschenrechner, der mathematische Ausdrücke berechnet."""

    def calculate(self, expression: str) -> float:
        """Berechnet einen mathematischen Ausdruck als String und überprüft auf ungültige Eingaben."""
        try:
            # Entfernt überflüssige Leerzeichen
            expression = expression.replace(" ", "")

            # Nur erlaubte Zeichen (Zahlen, Operatoren, Klammern) und Zahlen mit Vorzeichen
            if not re.match(r'^[0-9+\-*/().]+$', expression):
                raise ValueError("Ungültige Zeichen im Ausdruck.")

            # Überprüfung auf doppelte Operatoren, aber keine Vorzeichen wie -* oder -+
            # Hier wird jetzt auch sichergestellt, dass -3*-3 gültig bleibt
            if re.search(r'(?<!\d)[+\-*/]{2,}', expression):  # Erfasst auch doppelte Operatoren wie ++, --, **, etc.
                raise SyntaxError("Ungültige doppelte Operatoren im Ausdruck.")

            # Sicherstellen, dass Klammern ausgeglichen sind
            if expression.count("(") != expression.count(")"):
                raise ValueError("Fehlende oder zu viele Klammern.")

            # Sicherstellen, dass der Ausdruck nicht mit einem Operator beginnt oder endet,
            # aber das Minuszeichen als Teil der Zahl akzeptiert wird
            if re.match(r'^[*/]', expression) or re.match(r'[*/]$', expression):  # Weitere Kontrolle
                raise ValueError("Der Ausdruck darf nicht mit einem Operator beginnen oder enden.")

            # Evaluierung des mathematischen Ausdrucks mit eval()
            result = eval(expression, {"__builtins__": None}, {})

            # Sicherstellen, dass das Ergebnis numerisch ist
            if isinstance(result, (int, float)):
                return float(result)
            else:
                raise ValueError("Ungültiger Ausdruck.")

        except ZeroDivisionError:
            raise ZeroDivisionError("Division durch Null ist nicht erlaubt.")
        except (SyntaxError, TypeError, NameError) as e:
            raise ValueError(f"Ungültiger mathematischer Ausdruck: {str(e)}")
