from src.interfaces import IRomanConverter


def int_to_roman(num: int) -> str:
    """Konvertiert eine Dezimalzahl in eine römische Zahl."""
    val = [
        1000, 900, 500, 400,
        100, 90, 50, 40,
        10, 9, 5, 4,
        1
    ]
    syb = [
        "M", "CM", "D", "CD",
        "C", "XC", "L", "XL",
        "X", "IX", "V", "IV",
        "I"
    ]
    roman_num = ''
    i = 0
    while num > 0:
        for _ in range(num // val[i]):
            roman_num += syb[i]
            num -= val[i]
        i += 1
    return roman_num


class RomanConverter(IRomanConverter):
    def roman_to_int(self, s: str) -> int:
        """Konvertiert eine römische Zahl (String) in eine Dezimalzahl."""
        roman_values = {
            'I': 1, 'V': 5, 'X': 10, 'L': 50,
            'C': 100, 'D': 500, 'M': 1000
        }

        total = 0
        prev_value = 0

        # Iteriere über die Zeichen der römischen Zahl von rechts nach links
        for char in reversed(s.upper()):
            value = roman_values.get(char)
            if value is None:
                raise ValueError(f"Ungültiges Zeichen '{char}' in der römischen Zahl.")
            if value < prev_value:
                total -= value
            else:
                total += value
            prev_value = value

        return total
