# Testfälle für calculate(), Eingabe: str, Ausgabe: float/int

# 1️⃣ Additionstests
# Eingabe: "1+1" → Erwartete Ausgabe: 2
# Eingabe: "10+20" → Erwartete Ausgabe: 30
# Eingabe: "0+5" → Erwartete Ausgabe: 5
# Eingabe: "-3+7" → Erwartete Ausgabe: 4
# Eingabe: "2.5+2.5" → Erwartete Ausgabe: 5.0

# 2️⃣ Subtraktionstests
# Eingabe: "5-3" → Erwartete Ausgabe: 2
# Eingabe: "10-20" → Erwartete Ausgabe: -10
# Eingabe: "0-5" → Erwartete Ausgabe: -5
# Eingabe: "-3-7" → Erwartete Ausgabe: -10
# Eingabe: "2.5-1.5" → Erwartete Ausgabe: 1.0

# 3️⃣ Multiplikationstests
# Eingabe: "2*3" → Erwartete Ausgabe: 6
# Eingabe: "10*0" → Erwartete Ausgabe: 0
# Eingabe: "-2*5" → Erwartete Ausgabe: -10
# Eingabe: "3.5*2" → Erwartete Ausgabe: 7.0
# Eingabe: "-3*-3" → Erwartete Ausgabe: 9

# 4️⃣ Divisionstests
# Eingabe: "10/2" → Erwartete Ausgabe: 5
# Eingabe: "5/2" → Erwartete Ausgabe: 2.5
# Eingabe: "-6/3" → Erwartete Ausgabe: -2
# Eingabe: "7.5/2.5" → Erwartete Ausgabe: 3.0
# Eingabe: "5/0" → Erwartete Ausgabe: ZeroDivisionError (Fehlermeldung)

# 5️⃣ Komplexe Berechnungen
# Eingabe: "3+5*2" → Erwartete Ausgabe: 13 (Multiplikation vor Addition)
# Eingabe: "(3+5)*2" → Erwartete Ausgabe: 16 (Klammer zuerst)
# Eingabe: "10-4/2" → Erwartete Ausgabe: 8 (Division vor Subtraktion)
# Eingabe: "3+(2*5)-8/4" → Erwartete Ausgabe: 10 (Mehrere Operatoren)

# 6️⃣ Ungültige Eingaben
# Eingabe: "3++5" → Erwartete Ausgabe: SyntaxError (Fehlermeldung)
# Eingabe: "10*/2" → Erwartete Ausgabe: SyntaxError (Fehlermeldung)
# Eingabe: "abc" → Erwartete Ausgabe: ValueError (Fehlermeldung)


import unittest
from src.Calculator import Calculator

class TestCalculator(unittest.TestCase):
    def setUp(self):
        self.calc = Calculator()

    # Addition
    def test_addition(self):
        self.assertEqual(self.calc.calculate("1+1"), 2)
        self.assertEqual(self.calc.calculate("10+20"), 30)
        self.assertEqual(self.calc.calculate("0+5"), 5)
        self.assertEqual(self.calc.calculate("-3+7"), 4)
        self.assertEqual(self.calc.calculate("2.5+2.5"), 5.0)

    # Subtraktion
    def test_subtraction(self):
        self.assertEqual(self.calc.calculate("5-3"), 2)
        self.assertEqual(self.calc.calculate("10-20"), -10)
        self.assertEqual(self.calc.calculate("0-5"), -5)
        self.assertEqual(self.calc.calculate("-3-7"), -10)
        self.assertEqual(self.calc.calculate("2.5-1.5"), 1.0)

    # Multiplikation
    def test_multiplication(self):
        self.assertEqual(self.calc.calculate("2*3"), 6)
        self.assertEqual(self.calc.calculate("10*0"), 0)
        self.assertEqual(self.calc.calculate("-2*5"), -10)
        self.assertEqual(self.calc.calculate("3.5*2"), 7.0)
        self.assertEqual(self.calc.calculate("-3*-3"), 9)

    # Division
    def test_division(self):
        self.assertEqual(self.calc.calculate("10/2"), 5)
        self.assertEqual(self.calc.calculate("5/2"), 2.5)
        self.assertEqual(self.calc.calculate("-6/3"), -2)
        self.assertEqual(self.calc.calculate("7.5/2.5"), 3.0)

    # Division durch Null
    def test_division_by_zero(self):
        with self.assertRaises(ZeroDivisionError):
            self.calc.calculate("5/0")

    # Komplexe Berechnungen
    def test_complex_expressions(self):
        self.assertEqual(self.calc.calculate("3+5*2"), 13)  # Punkt-vor-Strich beachten
        self.assertEqual(self.calc.calculate("(3+5)*2"), 16)  # Klammer zuerst
        self.assertEqual(self.calc.calculate("10-4/2"), 8)  # Division vor Subtraktion
        self.assertEqual(self.calc.calculate("3+(2*5)-8/4"), 11.0)  #  11

    # Ungültige Eingaben
    """def test_invalid_expressions(self):
        with self.assertRaises(ValueError):
            self.calc.calculate("3++5")
        with self.assertRaises(ValueError):
            self.calc.calculate("10*/2")
        with self.assertRaises(ValueError):
            self.calc.calculate("abc")"""

if __name__ == "__main__":
    unittest.main()